<!-- Gitlab Specific Markdown guide https://docs.gitlab.com/ee/user/markdown.html -->
# T64-self-healing
This is a docker compose project that aim to instantiate the selfhealing piacere components for different purposes:
* production
* develop
* debug
* integration

## Prerequisites
In order to use this vagrantfile you need virtualbox and vagrant. 
* https://www.docker.com/ (latest tested version 20.10.8, build 3967b7d)
* https://docs.docker.com/compose/ (latest tested version 1.29.0, build 07737305)

## How to use
The docker-compose can be started in different ways depending your purpose. But in all the cases the first step is to download the repo. 

```bash
git clone https://git.code.tecnalia.com/piacere/private/t64-self-healing.git
cd t64-self-healing
````
NOTE: For develop and debug scenarios the repo relays in git submodules and therefore if we need to build the component images (the typical case) we need to user the "--recurse-submodules" option to download the submodules. Or issuing a "git submodule update --init --recursive" statement latter on.

After that we may need to change some of the variables at .env to secure the deployment or customize to our needs. Examples of variables that could be subject to change are:
```bash
export SERVER_HOST=somedomain
export HTTPS_PORT=8443
export ADMIN_PASSWORD=somestrongpassword

````
NOTE: Another posibility could be to edit the .env file directly, but beware of uploading secrets to git

### For production
By default the project is configured to support the production scenario where we relay in previously uploaded images to the artifactory. The file containing the default variables for this scenario is .env 
```bash
git clone https://git.code.tecnalia.com/piacere/private/t64-self-healing.git
cd t64-self-healing
export SERVER_HOST=basedomain.org
export HTTPS_PORT=8443
export ADMIN_PASSWORD=somestrongpassword
docker login optima-piacere-docker-dev.artifact.tecnalia.com -u user@domain.net -p repositoryToken 
docker-compose pull
docker-compose up -d --no-build --remove-orphans
echo to stop
docker-compose down --remove-orphans
echo if we what to remove also the persistent information
docker-compose down --volumes --remove-orphans
````
### For building
Another tipical scenario is to run the docker-compose for building the images after some updates. The file containing the default variables for this scenario is .env.build. 

```bash
git clone --recurse-submodules https://git.code.tecnalia.com/piacere/private/t64-self-healing.git
cd t64-self-healing
docker-compose --env-file .env.build build
docker login optima-piacere-docker-dev.artifact.tecnalia.com -u user@domain.net -p repositoryToken 
docker-compose --env-file .env.build push
````
NOTE: For this task as metioned before it is necessary to download the submodules

### For integration debug
Another tipical scenario is to run the docker-compose to test the whole solution during development or debug activities inside a development computer. The file containing the default variables for this scenario is .env.int. 

```bash
git clone --recurse-submodules https://git.code.tecnalia.com/piacere/private/t64-self-healing.git
cd t64-self-healing
export CERTIFICATE_SIGNING_KEY_PASSPHRASE=thesecretkeytodecryptthecaprivatekey
docker-compose --env-file .env.int build
docker-compose --env-file .env.int up -d --no-build --remove-orphans
docker-compose --env-file .env.int down --remove-orphans
echo if we what to remove also the persistent information
docker-compose down --env-file .env.int --volumes --remove-orphans
````

### For local development support
Another tipical scenario is to run the docker-compose to use some of the components to support the local development. ie.e databases, identity management systems, etc. The file containing the default variables for this scenario is .env.dev. 

```bash
git clone --recurse-submodules https://git.code.tecnalia.com/piacere/private/t64-self-healing.git
cd t64-self-healing
export CERTIFICATE_SIGNING_KEY_PASSPHRASE=thesecretkeytodecryptthecaprivatekey
docker-compose --env-file .env.dev build
docker-compose --env-file .env.dev up -d --no-build --remove-orphans
docker-compose --env-file .env.dev down --remove-orphans
echo if we what to remove also the persistent information
docker-compose down --env-file .env.dev --volumes --remove-orphans
````

## How to access
Once we sucessfully deploy the docker-compose, we will be able to access the services at (supposing SERVER_HOST=192.168.56.1.nip.io and HTTPS_PORT=8443):
* https://sh.192.168.56.1.nip.io:8443/ to access sh gateway for development support

Appart from those in case we are using the ".env.int" we will have access to aditional endpoints
* https://jhipster-registry.192.168.56.1.nip.io:8443/ to access jhipster registry 

And to those generic described at [development-services](git/deploy/development-services/README.md)
* https://traefik.192.168.56.1.nip.io:8443/ to access traefik dashboard 
* https://traefik.192.168.56.1.nip.io:8443/api/http/routers to access traefik api 
* https://portainer.192.168.56.1.nip.io:8443/ to access portainer 
* https://ca.192.168.56.1.nip.io:8443/ to access the tecnalia ca in case we use the tecnalia traefik
* https://auth.192.168.56.1.nip.io:8443/ to access keycloak
* ...

# Development

## import jhipster data model
jhipster import-jdl selfHealingService.jdl

## How to develop in local environment
deploy the docker-compose from root folder with "docker-compose -f docker-compose-local-dev.yaml up --build -d"
deploy service from selfHealingService folder "./mvnw -Pdev,api-docs,no-liquibase -Dskip-tests"
deploy gateway from selfHealingGateway folder  "./mvnw -Pdev,webapp,api-docs -Dskip-tests"   

#export mysql data
get container id with 
    "docker ps"
get script
    "docker exec -it 6a1 mysqldump -uroot --databases <databaseName> --tables <table1Name> <table2Name> <...> --skip-comments > /tmp/export.sql"
update startupscripts with data from export.sql 

## Liquibase
To regenerate the code and database like the first time:
- Delete json model in ".jhipster" folder
- Delete changelog xmls except initial_schema
- Set empty array in entities from ".yo-rc.json" file 
- Delete lastLiquibaseTimestamp property from .yo-rc.json file 
- Execute in microservice folder, "jhipster import selfHealingService.jdl"
After starting mysql service (docker-compose):
- Access mysql database and remove it.
- Deploy service with liquibase profile active
    "./mvnw -Pdev,api-docs -Dskip-tests"


# API usage
https://sh.piacere.esilab.org:8445/services/shbackend/api/self-healing/notify
