package com.piacere.selfhealing.backend.config;

import java.util.HashMap;
import java.util.Map;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import lombok.Getter;
import lombok.Setter;

import javax.annotation.PostConstruct;

@Configuration
@ConfigurationProperties(prefix = "kafka")
@Getter
@Setter
public class KafkaProperties {
	@Value("${kafka.bootstrap.servers:kafka:9092}")
    private String bootStrapServers;

    @Value("${kafka.polling.timeout:10000}")
    private Integer pollingTimeout;

    private Map<String, Map<String, Object>> consumer = new HashMap<>();

    private Map<String, Map<String, Object>> producer = new HashMap<>();

    @PostConstruct
    public void init() {
        for (String consumerKey : consumer.keySet()) {
            final Map<String, Object> properties = consumer.get(consumerKey);
            if (!properties.containsKey(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG)) {
                properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootStrapServers);
            }
        }

        for (String consumerKey : producer.keySet()) {
            final Map<String, Object> properties = producer.get(consumerKey);
            if (!properties.containsKey(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG)) {
                properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootStrapServers);
            }
        }
    }
}
