package com.piacere.selfhealing.backend.consumer;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.beans.factory.support.DefaultSingletonBeanRegistry;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.core.task.SimpleAsyncTaskExecutor;
import org.springframework.stereotype.Service;

import com.piacere.selfhealing.backend.config.KafkaProperties;
import com.piacere.selfhealing.backend.domain.SelfHealingMessage;
import com.piacere.selfhealing.backend.serde.DeserializationError;
import com.piacere.selfhealing.backend.service.SelfHealingMessageService;

import io.vavr.control.Either;

import java.util.Map;

@Service
public class SelfHealingMessageConsumer extends GenericConsumer<SelfHealingMessage> {
	private final Logger log = LoggerFactory.getLogger(SelfHealingMessageConsumer.class);
    private final SelfHealingMessageService selfHealingMessageService;

    private final ApplicationContext applicationContext;

    public SelfHealingMessageConsumer(
        @Value("${kafka.topic.selfHealingMessage}") final String topicName,
        final KafkaProperties kafkaProperties,
        SelfHealingMessageService selfHealingMessageService,
        ApplicationContext applicationContext
    ) {
        super(topicName, kafkaProperties.getConsumer().get("selfHealingMessage"), kafkaProperties.getPollingTimeout());
        this.selfHealingMessageService = selfHealingMessageService;
        this.applicationContext = applicationContext;
    }

    @Override
    protected void handleMessage(final ConsumerRecord<String, Either<DeserializationError, SelfHealingMessage>> record) {
        final Either<DeserializationError, SelfHealingMessage> value = record.value();
        if (value == null) {
            log.error("Null value in record {}", record);
            return;
        }
        if (value.isLeft()) {
            log.error("Deserialization record failure: {}", value.getLeft());
            return;
        }
        log.debug("----SELF-HEALING PROCESS STARTED------------------------------");
        try {
            this.selfHealingMessageService.processMessage(value.get());
        } catch (Exception e) {
            e.printStackTrace();
        }

        log.debug("----SELF-HEALING PROCESS FINISHED------------------------------");
    }

    @Bean
    public void executeKafkaControlsSecurityMetricRunner() {
        new SimpleAsyncTaskExecutor().execute(this);
    }

}
