package com.piacere.selfhealing.backend.domain;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import javax.persistence.*;
import javax.validation.constraints.*;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * deployment of self-healing\n\n@author Tecnalia
 */
@ApiModel(description = "deployment of self-healing\n\n@author Tecnalia")
@Entity
@Table(name = "deployment")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class Deployment implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    /**
     * deployment id
     */
    @ApiModelProperty(value = "deployment id", required = true)
    @NotNull
    @Column(name = "deployment_id", nullable = false, unique = true)
    private String deploymentId;

    /**
     * information about the deployment zip bundle
     */
    @ApiModelProperty(value = "information about the deployment zip bundle", required = true)
    @Lob
    @Column(name = "bundle", nullable = false)
    private byte[] bundle;

    @NotNull
    @Column(name = "bundle_content_type", nullable = false)
    private String bundleContentType;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Deployment id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getDeploymentId() {
        return this.deploymentId;
    }

    public Deployment deploymentId(String deploymentId) {
        this.setDeploymentId(deploymentId);
        return this;
    }

    public void setDeploymentId(String deploymentId) {
        this.deploymentId = deploymentId;
    }

    public byte[] getBundle() {
        return this.bundle;
    }

    public Deployment bundle(byte[] bundle) {
        this.setBundle(bundle);
        return this;
    }

    public void setBundle(byte[] bundle) {
        this.bundle = bundle;
    }

    public String getBundleContentType() {
        return this.bundleContentType;
    }

    public Deployment bundleContentType(String bundleContentType) {
        this.bundleContentType = bundleContentType;
        return this;
    }

    public void setBundleContentType(String bundleContentType) {
        this.bundleContentType = bundleContentType;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Deployment)) {
            return false;
        }
        return id != null && id.equals(((Deployment) o).id);
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Deployment{" +
            "id=" + getId() +
            ", deploymentId='" + getDeploymentId() + "'" +
            ", bundle='" + getBundle() + "'" +
            ", bundleContentType='" + getBundleContentType() + "'" +
            "}";
    }
}
