package com.piacere.selfhealing.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.piacere.selfhealing.backend.domain.enumeration.Status;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.time.Instant;
import javax.persistence.*;
import javax.validation.constraints.*;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * Messages received definition https:\n\n@author Tecnalia
 */
@ApiModel(description = "Messages received definition https:\n\n@author Tecnalia")
@Entity
@Table(name = "self_healing_message")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class SelfHealingMessage implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    /**
     * message timestamp
     */
    @ApiModelProperty(value = "message timestamp")
    @Column(name = "timestamp")
    private Instant timestamp;

    /**
     * message origin component
     */
    @ApiModelProperty(value = "message origin component", required = true)
    @NotNull
    @Column(name = "origin", nullable = false)
    private String origin;

    /**
     * id of app related to message
     */
    @ApiModelProperty(value = "id of app related to message")
    @Column(name = "deployment_id")
    private String deploymentId;

    /**
     * status of the message
     */
    @ApiModelProperty(value = "status of the message")
    @Enumerated(EnumType.STRING)
    @Column(name = "status")
    private Status status;

    /**
     * error description if status error
     */
    @ApiModelProperty(value = "error description if status error")
    @Lob
    @Column(name = "error")
    private String error;

    @ManyToOne
    @JsonIgnoreProperties(value = { "strategy", "deployment" }, allowSetters = true)
    private EventType eventType;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public SelfHealingMessage id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Instant getTimestamp() {
        return this.timestamp;
    }

    public SelfHealingMessage timestamp(Instant timestamp) {
        this.setTimestamp(timestamp);
        return this;
    }

    public void setTimestamp(Instant timestamp) {
        this.timestamp = timestamp;
    }

    public String getOrigin() {
        return this.origin;
    }

    public SelfHealingMessage origin(String origin) {
        this.setOrigin(origin);
        return this;
    }

    public void setOrigin(String origin) {
        this.origin = origin;
    }

    public String getDeploymentId() {
        return this.deploymentId;
    }

    public SelfHealingMessage deploymentId(String deploymentId) {
        this.setDeploymentId(deploymentId);
        return this;
    }

    public void setDeploymentId(String deploymentId) {
        this.deploymentId = deploymentId;
    }

    public Status getStatus() {
        return this.status;
    }

    public SelfHealingMessage status(Status status) {
        this.setStatus(status);
        return this;
    }

    public void setStatus(Status status) {
        this.status = status;
    }

    public String getError() {
        return this.error;
    }

    public SelfHealingMessage error(String error) {
        this.setError(error);
        return this;
    }

    public void setError(String error) {
        this.error = error;
    }

    public EventType getEventType() {
        return this.eventType;
    }

    public void setEventType(EventType eventType) {
        this.eventType = eventType;
    }

    public SelfHealingMessage eventType(EventType eventType) {
        this.setEventType(eventType);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SelfHealingMessage)) {
            return false;
        }
        return id != null && id.equals(((SelfHealingMessage) o).id);
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "SelfHealingMessage{" +
            "id=" + getId() +
            ", timestamp='" + getTimestamp() + "'" +
            ", origin='" + getOrigin() + "'" +
            ", deploymentId='" + getDeploymentId() + "'" +
            ", status='" + getStatus() + "'" +
            ", error='" + getError() + "'" +
            "}";
    }
}
