package com.piacere.selfhealing.backend.domain;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import javax.persistence.*;
import javax.validation.constraints.*;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * strategys managed by self-healing\n\n@author Tecnalia
 */
@ApiModel(description = "strategys managed by self-healing\n\n@author Tecnalia")
@Entity
@Table(name = "strategy")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class Strategy implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    /**
     * strategy code
     */
    @ApiModelProperty(value = "strategy code", required = true)
    @NotNull
    @Column(name = "code", nullable = false, unique = true)
    private String code;

    /**
     * strategy description
     */
    @ApiModelProperty(value = "strategy description")
    @Column(name = "description")
    private String description;

    /**
     * business process modelling annotation to propose
     */
    @ApiModelProperty(value = "business process modelling annotation to propose")
    @Lob
    @Column(name = "configuration")
    private String configuration;

    @ManyToOne
    private Deployment deployment;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Strategy id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCode() {
        return this.code;
    }

    public Strategy code(String code) {
        this.setCode(code);
        return this;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getDescription() {
        return this.description;
    }

    public Strategy description(String description) {
        this.setDescription(description);
        return this;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getConfiguration() {
        return this.configuration;
    }

    public Strategy configuration(String configuration) {
        this.setConfiguration(configuration);
        return this;
    }

    public void setConfiguration(String configuration) {
        this.configuration = configuration;
    }

    public Deployment getDeployment() {
        return this.deployment;
    }

    public void setDeployment(Deployment deployment) {
        this.deployment = deployment;
    }

    public Strategy deployment(Deployment deployment) {
        this.setDeployment(deployment);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Strategy)) {
            return false;
        }
        return id != null && id.equals(((Strategy) o).id);
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Strategy{" +
            "id=" + getId() +
            ", code='" + getCode() + "'" +
            ", description='" + getDescription() + "'" +
            ", configuration='" + getConfiguration() + "'" +
            "}";
    }
}
