package com.piacere.selfhealing.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.time.Instant;
import javax.persistence.*;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * strategyApplication of self-healing\n\n@author Tecnalia
 */
@ApiModel(description = "strategyApplication of self-healing\n\n@author Tecnalia")
@Entity
@Table(name = "strategy_application")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class StrategyApplication implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    /**
     * strategyApplication timestamp
     */
    @ApiModelProperty(value = "strategyApplication timestamp")
    @Column(name = "timestamp")
    private Instant timestamp;

    /**
     * json with summary of strategyApplication
     */
    @ApiModelProperty(value = "json with summary of strategyApplication")
    @Lob
    @Column(name = "summary")
    private String summary;

    /**
     * optional bundle with strategyApplication resources
     */
    @ApiModelProperty(value = "optional bundle with strategyApplication resources")
    @Lob
    @Column(name = "bundle")
    private byte[] bundle;

    @Column(name = "bundle_content_type")
    private String bundleContentType;

    @ManyToOne
    @JsonIgnoreProperties(value = { "deployment" }, allowSetters = true)
    private Strategy strategy;

    @ManyToOne
    private Deployment deployment;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public StrategyApplication id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Instant getTimestamp() {
        return this.timestamp;
    }

    public StrategyApplication timestamp(Instant timestamp) {
        this.setTimestamp(timestamp);
        return this;
    }

    public void setTimestamp(Instant timestamp) {
        this.timestamp = timestamp;
    }

    public String getSummary() {
        return this.summary;
    }

    public StrategyApplication summary(String summary) {
        this.setSummary(summary);
        return this;
    }

    public void setSummary(String summary) {
        this.summary = summary;
    }

    public byte[] getBundle() {
        return this.bundle;
    }

    public StrategyApplication bundle(byte[] bundle) {
        this.setBundle(bundle);
        return this;
    }

    public void setBundle(byte[] bundle) {
        this.bundle = bundle;
    }

    public String getBundleContentType() {
        return this.bundleContentType;
    }

    public StrategyApplication bundleContentType(String bundleContentType) {
        this.bundleContentType = bundleContentType;
        return this;
    }

    public void setBundleContentType(String bundleContentType) {
        this.bundleContentType = bundleContentType;
    }

    public Strategy getStrategy() {
        return this.strategy;
    }

    public void setStrategy(Strategy strategy) {
        this.strategy = strategy;
    }

    public StrategyApplication strategy(Strategy strategy) {
        this.setStrategy(strategy);
        return this;
    }

    public Deployment getDeployment() {
        return this.deployment;
    }

    public void setDeployment(Deployment deployment) {
        this.deployment = deployment;
    }

    public StrategyApplication deployment(Deployment deployment) {
        this.setDeployment(deployment);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof StrategyApplication)) {
            return false;
        }
        return id != null && id.equals(((StrategyApplication) o).id);
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "StrategyApplication{" +
            "id=" + getId() +
            ", timestamp='" + getTimestamp() + "'" +
            ", summary='" + getSummary() + "'" +
            ", bundle='" + getBundle() + "'" +
            ", bundleContentType='" + getBundleContentType() + "'" +
            "}";
    }
}
