package com.piacere.selfhealing.backend.producer;

import com.piacere.selfhealing.backend.config.KafkaProperties;
import com.piacere.selfhealing.backend.domain.SelfHealingMessage;

import javax.annotation.PostConstruct;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Map;

@Service
public class SelfHealingMessageProducer {
	enum ReferenceTomProducerKey {
        SELFHEALINGMESSAGE_NOTIFICATION,
    }

    private final Logger log = LoggerFactory.getLogger(SelfHealingMessageProducer.class);

    private final KafkaProducer<String, SelfHealingMessage> kafkaProducer;

    private final String topicName;

    public SelfHealingMessageProducer(
        @Value("${kafka.topic.selfHealingMessage}") final String topicName,
        final KafkaProperties kafkaProperties
    ) {
        this.topicName = topicName;
        this.kafkaProducer = new KafkaProducer<>(kafkaProperties.getProducer().get("selfHealingMessage"));
    }

    @PostConstruct
    public void init() {
        Runtime.getRuntime().addShutdownHook(new Thread(this::shutdown));
    }

    public void notify(final SelfHealingMessage message) {
        final ProducerRecord<String, SelfHealingMessage> record = new ProducerRecord<>(
            topicName,
            ReferenceTomProducerKey.SELFHEALINGMESSAGE_NOTIFICATION.toString(),
            message
        );
        try {
            log.info("Creating asynchronously a SelfHealingMessage record to topic: '" + topicName + "'");
            kafkaProducer.send(record);
        } catch (final Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public void shutdown() {
        log.info("Shutdown Kafka producer");
        kafkaProducer.close();
    }

}
