package com.piacere.selfhealing.backend.serde;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.util.StdDateFormat;
import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.module.paramnames.ParameterNamesModule;
import com.piacere.selfhealing.backend.domain.SelfHealingMessage;

import io.vavr.control.Either;
import java.io.IOException;
import org.apache.kafka.common.serialization.Deserializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SelfHealingMessageDeserializer implements Deserializer<Either<DeserializationError, SelfHealingMessage>> {

    private final Logger log = LoggerFactory.getLogger(SelfHealingMessageDeserializer.class);

    private final ObjectMapper objectMapper;

    public SelfHealingMessageDeserializer() {
        this.objectMapper =
            new ObjectMapper()
                .registerModule(new ParameterNamesModule())
                .registerModule(new Jdk8Module())
                .registerModule(new JavaTimeModule())
                .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
                .setDateFormat(new StdDateFormat());
    }

    @Override
    public Either<DeserializationError, SelfHealingMessage> deserialize(final String topicName, final byte[] data) {
        try {
            final SelfHealingMessage value = objectMapper.readValue(data, SelfHealingMessage.class);
            return Either.right(value);
        } catch (final IOException e) {
            return Either.left(new DeserializationError(data, e));
        }
    }
}
