package com.piacere.selfhealing.backend.serde;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.util.StdDateFormat;
import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.module.paramnames.ParameterNamesModule;
import com.piacere.selfhealing.backend.domain.SelfHealingMessage;

import org.apache.kafka.common.errors.SerializationException;
import org.apache.kafka.common.serialization.Serializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SelfHealingMessageSerializer implements Serializer<SelfHealingMessage> {

    private final Logger log = LoggerFactory.getLogger(SelfHealingMessageSerializer.class);

    private final ObjectMapper objectMapper;

    public SelfHealingMessageSerializer() {
        this.objectMapper =
            new ObjectMapper()
                .registerModule(new ParameterNamesModule())
                .registerModule(new Jdk8Module())
                .registerModule(new JavaTimeModule())
                .setDateFormat(new StdDateFormat());
    }

    @Override
    public byte[] serialize(final String topicName, final SelfHealingMessage selfHealingMessage) {
        if (selfHealingMessage == null) {
            return null;
        }
        try {
            return objectMapper.writeValueAsBytes(selfHealingMessage);
        } catch (final JsonProcessingException e) {
            throw new SerializationException(e.getMessage(), e);
        }
    }
}
