package com.piacere.selfhealing.backend.service;

import com.piacere.selfhealing.backend.config.IcgProperties;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.UUID;

@Service
public class IcgService {
    // PiacereIDEIntegration.KR03
    private final Logger log = LoggerFactory.getLogger(IopService.class);

    private IcgProperties icgProperties;

    public IcgService(IcgProperties icgProperties) {
        this.icgProperties = icgProperties;
    }
    public void generateIac(Path bundlePath, Path resultBundlePath)  {
        log.info("gathering iac {}", bundlePath);
        String url = icgProperties.getUrl() + "/iac/files/upload";

        MultipartEntityBuilder builder = MultipartEntityBuilder.create();
        builder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
        builder.addBinaryBody("file", bundlePath.toFile(), ContentType.create("application/x-zip-compressed"), UUID.randomUUID().toString()+".zip");
        builder.setBoundary("---Content Boundary");

        try (CloseableHttpClient httpClient = HttpClientBuilder.create().build()) {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Accept", "application/json");
            httpPost.setHeader("Content-type", "multipart/form-data; boundary=\"---Content Boundary\"");
            httpPost.setEntity(builder.build());
            try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
                // if response status code is not 200, throw exception
                if (response.getStatusLine().getStatusCode() != 200) {
                    throw new RuntimeException("Failed : HTTP error code : "
                        + response.getStatusLine().getStatusCode());
                }

                FileOutputStream fileOutputStream = new FileOutputStream(resultBundlePath.toFile());

                int bytesRead = -1;
                byte[] buffer = new byte[4096];
                while ((bytesRead = response.getEntity().getContent().read(buffer)) != -1 ) {
                    fileOutputStream.write(buffer, 0, bytesRead);
                }
                fileOutputStream.close();
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
