package com.piacere.selfhealing.backend.service;

import com.piacere.selfhealing.backend.config.IopProperties;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.nio.file.Path;

@Service
public class IopService {
    //PiacereIDEIntegration.KR09\src\eu\piacere\ide\toolIntegration\kr09\commandHandlers\OptimizerCommandHandler.java
    private final Logger log = LoggerFactory.getLogger(IopService.class);

    private IopProperties iopProperties;

    public IopService(IopProperties iopProperties) {
        this.iopProperties = iopProperties;
    }

    public String optimize(Path domlPath) {
        log.info("Optimizing {}", domlPath);
        String url = iopProperties.getUrl() + "/api/optimize";

        InputStream inputStream = null;
        try {
            inputStream = Files.newInputStream(domlPath);
        } catch (IOException e) {
            e.printStackTrace();
            new RuntimeException(e);
        }

        String content = null;
        try {
            content = new String(inputStream.readAllBytes());
        } catch (IOException e) {
            e.printStackTrace();
            new RuntimeException(e);
        }

        StringEntity entity = null;
        try {
            entity = new StringEntity(content);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
        try (CloseableHttpClient httpClient = HttpClientBuilder.create().build()) {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setEntity(entity);
            httpPost.setHeader("Accept", "application/json");
            try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
                // if response status code is not 200, throw exception
                if (response.getStatusLine().getStatusCode() != 200) {
                    throw new RuntimeException("Failed : HTTP error code : "
                            + response.getStatusLine().getStatusCode());
                }
                HttpEntity responseEntity = response.getEntity();
                return EntityUtils.toString(responseEntity);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
