package com.piacere.selfhealing.backend.service;

import com.google.gson.JsonObject;
import com.piacere.selfhealing.backend.config.PrcProperties;
import com.piacere.selfhealing.backend.domain.Deployment;
import eu.piacere.prc.client.ApiClient;
import eu.piacere.prc.client.ApiException;
import eu.piacere.prc.client.Configuration;
import eu.piacere.prc.client.api.DefaultApi;
import eu.piacere.prc.client.model.SelfHealing;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

@Service
public class PrcService {
    private final Logger log = LoggerFactory.getLogger(this.getClass());
    private final PrcProperties prcProperties;

    public PrcService(PrcProperties iemProperties) {
        this.prcProperties = iemProperties;
    }

    public void executeAnsiblePlaybook(String deploymentId, String playbook) {
        log.info("Executing Ansible Playbook");
        ApiClient prcApiClient = getPrcApiClient();

        DefaultApi deploymentsApi = new DefaultApi(prcApiClient);
        SelfHealing selfHealing = new SelfHealing();
        selfHealing.setPlaybook(playbook);
        try {
            deploymentsApi.selfHealingDeploymentsDeploymentIdSelfHealingPost(deploymentId, selfHealing);
        } catch (ApiException e) {
            log.error("Error executing Ansible Playbook", e);
            e.printStackTrace();
        }
    }
    private ApiClient getPrcApiClient() {
        ApiClient defaultClient = Configuration.getDefaultApiClient();
        if (log.isDebugEnabled()) {
            defaultClient.setDebugging(true);
        }

        defaultClient.setBasePath(prcProperties.getUrl());

        return defaultClient;
    }

    public void executeRedeploy(Deployment deployment) {
        log.info("Executing Redeploy");
        ApiClient prcApiClient = getPrcApiClient();

        DefaultApi deploymentsApi = new DefaultApi(prcApiClient);

        try {
            deploymentsApi.redeployDeploymentsDeploymentIdRedeployPost(deployment.getDeploymentId());
        } catch (ApiException e) {
            log.error("Error executing Redeploy", e);
            e.printStackTrace();
        }
    }
}
