jest.mock('@angular/router');

import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, Router } from '@angular/router';
import { of } from 'rxjs';

import { IDeployment, Deployment } from '../deployment.model';
import { DeploymentService } from '../service/deployment.service';

import { DeploymentRoutingResolveService } from './deployment-routing-resolve.service';

describe('Service Tests', () => {
  describe('Deployment routing resolve service', () => {
    let mockRouter: Router;
    let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
    let routingResolveService: DeploymentRoutingResolveService;
    let service: DeploymentService;
    let resultDeployment: IDeployment | undefined;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        providers: [Router, ActivatedRouteSnapshot],
      });
      mockRouter = TestBed.inject(Router);
      mockActivatedRouteSnapshot = TestBed.inject(ActivatedRouteSnapshot);
      routingResolveService = TestBed.inject(DeploymentRoutingResolveService);
      service = TestBed.inject(DeploymentService);
      resultDeployment = undefined;
    });

    describe('resolve', () => {
      it('should return IDeployment returned by find', () => {
        // GIVEN
        service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultDeployment = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultDeployment).toEqual({ id: 123 });
      });

      it('should return new IDeployment if id is not provided', () => {
        // GIVEN
        service.find = jest.fn();
        mockActivatedRouteSnapshot.params = {};

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultDeployment = result;
        });

        // THEN
        expect(service.find).not.toBeCalled();
        expect(resultDeployment).toEqual(new Deployment());
      });

      it('should route to 404 page if data not found in server', () => {
        // GIVEN
        jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as Deployment })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultDeployment = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultDeployment).toEqual(undefined);
        expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
      });
    });
  });
});
