import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { IDeployment, Deployment } from '../deployment.model';
import { DeploymentService } from '../service/deployment.service';

@Injectable({ providedIn: 'root' })
export class DeploymentRoutingResolveService implements Resolve<IDeployment> {
  constructor(protected service: DeploymentService, protected router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IDeployment> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        mergeMap((deployment: HttpResponse<Deployment>) => {
          if (deployment.body) {
            return of(deployment.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new Deployment());
  }
}
