jest.mock('@angular/router');

import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { of, Subject } from 'rxjs';

import { DeploymentService } from '../service/deployment.service';
import { IDeployment, Deployment } from '../deployment.model';

import { DeploymentUpdateComponent } from './deployment-update.component';

describe('Component Tests', () => {
  describe('Deployment Management Update Component', () => {
    let comp: DeploymentUpdateComponent;
    let fixture: ComponentFixture<DeploymentUpdateComponent>;
    let activatedRoute: ActivatedRoute;
    let deploymentService: DeploymentService;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        declarations: [DeploymentUpdateComponent],
        providers: [FormBuilder, ActivatedRoute],
      })
        .overrideTemplate(DeploymentUpdateComponent, '')
        .compileComponents();

      fixture = TestBed.createComponent(DeploymentUpdateComponent);
      activatedRoute = TestBed.inject(ActivatedRoute);
      deploymentService = TestBed.inject(DeploymentService);

      comp = fixture.componentInstance;
    });

    describe('ngOnInit', () => {
      it('Should update editForm', () => {
        const deployment: IDeployment = { id: 456 };

        activatedRoute.data = of({ deployment });
        comp.ngOnInit();

        expect(comp.editForm.value).toEqual(expect.objectContaining(deployment));
      });
    });

    describe('save', () => {
      it('Should call update service on save for existing entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<Deployment>>();
        const deployment = { id: 123 };
        jest.spyOn(deploymentService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ deployment });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: deployment }));
        saveSubject.complete();

        // THEN
        expect(comp.previousState).toHaveBeenCalled();
        expect(deploymentService.update).toHaveBeenCalledWith(deployment);
        expect(comp.isSaving).toEqual(false);
      });

      it('Should call create service on save for new entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<Deployment>>();
        const deployment = new Deployment();
        jest.spyOn(deploymentService, 'create').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ deployment });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: deployment }));
        saveSubject.complete();

        // THEN
        expect(deploymentService.create).toHaveBeenCalledWith(deployment);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).toHaveBeenCalled();
      });

      it('Should set isSaving to false on error', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<Deployment>>();
        const deployment = { id: 123 };
        jest.spyOn(deploymentService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ deployment });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.error('This is an error!');

        // THEN
        expect(deploymentService.update).toHaveBeenCalledWith(deployment);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).not.toHaveBeenCalled();
      });
    });
  });
});
