jest.mock('@ng-bootstrap/ng-bootstrap');

import { ComponentFixture, TestBed, inject, fakeAsync, tick } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { of } from 'rxjs';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';

import { SelfHealingMessageService } from '../service/self-healing-message.service';

import { SelfHealingMessageDeleteDialogComponent } from './self-healing-message-delete-dialog.component';

describe('Component Tests', () => {
  describe('SelfHealingMessage Management Delete Component', () => {
    let comp: SelfHealingMessageDeleteDialogComponent;
    let fixture: ComponentFixture<SelfHealingMessageDeleteDialogComponent>;
    let service: SelfHealingMessageService;
    let mockActiveModal: NgbActiveModal;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        declarations: [SelfHealingMessageDeleteDialogComponent],
        providers: [NgbActiveModal],
      })
        .overrideTemplate(SelfHealingMessageDeleteDialogComponent, '')
        .compileComponents();
      fixture = TestBed.createComponent(SelfHealingMessageDeleteDialogComponent);
      comp = fixture.componentInstance;
      service = TestBed.inject(SelfHealingMessageService);
      mockActiveModal = TestBed.inject(NgbActiveModal);
    });

    describe('confirmDelete', () => {
      it('Should call delete service on confirmDelete', inject(
        [],
        fakeAsync(() => {
          // GIVEN
          jest.spyOn(service, 'delete').mockReturnValue(of(new HttpResponse({})));

          // WHEN
          comp.confirmDelete(123);
          tick();

          // THEN
          expect(service.delete).toHaveBeenCalledWith(123);
          expect(mockActiveModal.close).toHaveBeenCalledWith('deleted');
        })
      ));

      it('Should not call delete service on clear', () => {
        // GIVEN
        jest.spyOn(service, 'delete');

        // WHEN
        comp.cancel();

        // THEN
        expect(service.delete).not.toHaveBeenCalled();
        expect(mockActiveModal.close).not.toHaveBeenCalled();
        expect(mockActiveModal.dismiss).toHaveBeenCalled();
      });
    });
  });
});
