import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { ISelfHealingMessage } from '../self-healing-message.model';
import { SelfHealingMessageService } from '../service/self-healing-message.service';
import { SelfHealingMessageDeleteDialogComponent } from '../delete/self-healing-message-delete-dialog.component';
import { DataUtils } from 'app/core/util/data-util.service';

@Component({
  selector: 'jhi-self-healing-message',
  templateUrl: './self-healing-message.component.html',
})
export class SelfHealingMessageComponent implements OnInit {
  selfHealingMessages?: ISelfHealingMessage[];
  isLoading = false;

  constructor(
    protected selfHealingMessageService: SelfHealingMessageService,
    protected dataUtils: DataUtils,
    protected modalService: NgbModal
  ) {}

  loadAll(): void {
    this.isLoading = true;

    this.selfHealingMessageService.query().subscribe(
      (res: HttpResponse<ISelfHealingMessage[]>) => {
        this.isLoading = false;
        this.selfHealingMessages = res.body ?? [];
      },
      () => {
        this.isLoading = false;
      }
    );
  }

  ngOnInit(): void {
    this.loadAll();
  }

  trackId(index: number, item: ISelfHealingMessage): number {
    return item.id!;
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    return this.dataUtils.openFile(base64String, contentType);
  }

  delete(selfHealingMessage: ISelfHealingMessage): void {
    const modalRef = this.modalService.open(SelfHealingMessageDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.selfHealingMessage = selfHealingMessage;
    // unsubscribe not needed because closed completes on modal close
    modalRef.closed.subscribe(reason => {
      if (reason === 'deleted') {
        this.loadAll();
      }
    });
  }
}
