import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { ISelfHealingMessage, SelfHealingMessage } from '../self-healing-message.model';
import { SelfHealingMessageService } from '../service/self-healing-message.service';

@Injectable({ providedIn: 'root' })
export class SelfHealingMessageRoutingResolveService implements Resolve<ISelfHealingMessage> {
  constructor(protected service: SelfHealingMessageService, protected router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<ISelfHealingMessage> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        mergeMap((selfHealingMessage: HttpResponse<SelfHealingMessage>) => {
          if (selfHealingMessage.body) {
            return of(selfHealingMessage.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new SelfHealingMessage());
  }
}
