import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { IStrategyApplication } from '../strategy-application.model';
import { StrategyApplicationService } from '../service/strategy-application.service';
import { StrategyApplicationDeleteDialogComponent } from '../delete/strategy-application-delete-dialog.component';
import { DataUtils } from 'app/core/util/data-util.service';

@Component({
  selector: 'jhi-strategy-application',
  templateUrl: './strategy-application.component.html',
})
export class StrategyApplicationComponent implements OnInit {
  strategyApplications?: IStrategyApplication[];
  isLoading = false;

  constructor(
    protected strategyApplicationService: StrategyApplicationService,
    protected dataUtils: DataUtils,
    protected modalService: NgbModal
  ) {}

  loadAll(): void {
    this.isLoading = true;

    this.strategyApplicationService.query().subscribe(
      (res: HttpResponse<IStrategyApplication[]>) => {
        this.isLoading = false;
        this.strategyApplications = res.body ?? [];
      },
      () => {
        this.isLoading = false;
      }
    );
  }

  ngOnInit(): void {
    this.loadAll();
  }

  trackId(index: number, item: IStrategyApplication): number {
    return item.id!;
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    return this.dataUtils.openFile(base64String, contentType);
  }

  delete(strategyApplication: IStrategyApplication): void {
    const modalRef = this.modalService.open(StrategyApplicationDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.strategyApplication = strategyApplication;
    // unsubscribe not needed because closed completes on modal close
    modalRef.closed.subscribe(reason => {
      if (reason === 'deleted') {
        this.loadAll();
      }
    });
  }
}
