jest.mock('@angular/router');

import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, Router } from '@angular/router';
import { of } from 'rxjs';

import { IStrategyApplication, StrategyApplication } from '../strategy-application.model';
import { StrategyApplicationService } from '../service/strategy-application.service';

import { StrategyApplicationRoutingResolveService } from './strategy-application-routing-resolve.service';

describe('Service Tests', () => {
  describe('StrategyApplication routing resolve service', () => {
    let mockRouter: Router;
    let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
    let routingResolveService: StrategyApplicationRoutingResolveService;
    let service: StrategyApplicationService;
    let resultStrategyApplication: IStrategyApplication | undefined;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        providers: [Router, ActivatedRouteSnapshot],
      });
      mockRouter = TestBed.inject(Router);
      mockActivatedRouteSnapshot = TestBed.inject(ActivatedRouteSnapshot);
      routingResolveService = TestBed.inject(StrategyApplicationRoutingResolveService);
      service = TestBed.inject(StrategyApplicationService);
      resultStrategyApplication = undefined;
    });

    describe('resolve', () => {
      it('should return IStrategyApplication returned by find', () => {
        // GIVEN
        service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultStrategyApplication = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultStrategyApplication).toEqual({ id: 123 });
      });

      it('should return new IStrategyApplication if id is not provided', () => {
        // GIVEN
        service.find = jest.fn();
        mockActivatedRouteSnapshot.params = {};

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultStrategyApplication = result;
        });

        // THEN
        expect(service.find).not.toBeCalled();
        expect(resultStrategyApplication).toEqual(new StrategyApplication());
      });

      it('should route to 404 page if data not found in server', () => {
        // GIVEN
        jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as StrategyApplication })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultStrategyApplication = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultStrategyApplication).toEqual(undefined);
        expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
      });
    });
  });
});
