import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { IStrategyApplication, StrategyApplication } from '../strategy-application.model';
import { StrategyApplicationService } from '../service/strategy-application.service';

@Injectable({ providedIn: 'root' })
export class StrategyApplicationRoutingResolveService implements Resolve<IStrategyApplication> {
  constructor(protected service: StrategyApplicationService, protected router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IStrategyApplication> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        mergeMap((strategyApplication: HttpResponse<StrategyApplication>) => {
          if (strategyApplication.body) {
            return of(strategyApplication.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new StrategyApplication());
  }
}
