import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { IStrategy } from '../strategy.model';
import { StrategyService } from '../service/strategy.service';
import { StrategyDeleteDialogComponent } from '../delete/strategy-delete-dialog.component';
import { DataUtils } from 'app/core/util/data-util.service';

@Component({
  selector: 'jhi-strategy',
  templateUrl: './strategy.component.html',
})
export class StrategyComponent implements OnInit {
  strategies?: IStrategy[];
  isLoading = false;

  constructor(protected strategyService: StrategyService, protected dataUtils: DataUtils, protected modalService: NgbModal) {}

  loadAll(): void {
    this.isLoading = true;

    this.strategyService.query().subscribe(
      (res: HttpResponse<IStrategy[]>) => {
        this.isLoading = false;
        this.strategies = res.body ?? [];
      },
      () => {
        this.isLoading = false;
      }
    );
  }

  ngOnInit(): void {
    this.loadAll();
  }

  trackId(index: number, item: IStrategy): number {
    return item.id!;
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    return this.dataUtils.openFile(base64String, contentType);
  }

  delete(strategy: IStrategy): void {
    const modalRef = this.modalService.open(StrategyDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.strategy = strategy;
    // unsubscribe not needed because closed completes on modal close
    modalRef.closed.subscribe(reason => {
      if (reason === 'deleted') {
        this.loadAll();
      }
    });
  }
}
