jest.mock('@angular/router');

import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, Router } from '@angular/router';
import { of } from 'rxjs';

import { IStrategy, Strategy } from '../strategy.model';
import { StrategyService } from '../service/strategy.service';

import { StrategyRoutingResolveService } from './strategy-routing-resolve.service';

describe('Service Tests', () => {
  describe('Strategy routing resolve service', () => {
    let mockRouter: Router;
    let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
    let routingResolveService: StrategyRoutingResolveService;
    let service: StrategyService;
    let resultStrategy: IStrategy | undefined;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        providers: [Router, ActivatedRouteSnapshot],
      });
      mockRouter = TestBed.inject(Router);
      mockActivatedRouteSnapshot = TestBed.inject(ActivatedRouteSnapshot);
      routingResolveService = TestBed.inject(StrategyRoutingResolveService);
      service = TestBed.inject(StrategyService);
      resultStrategy = undefined;
    });

    describe('resolve', () => {
      it('should return IStrategy returned by find', () => {
        // GIVEN
        service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultStrategy = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultStrategy).toEqual({ id: 123 });
      });

      it('should return new IStrategy if id is not provided', () => {
        // GIVEN
        service.find = jest.fn();
        mockActivatedRouteSnapshot.params = {};

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultStrategy = result;
        });

        // THEN
        expect(service.find).not.toBeCalled();
        expect(resultStrategy).toEqual(new Strategy());
      });

      it('should route to 404 page if data not found in server', () => {
        // GIVEN
        jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as Strategy })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultStrategy = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultStrategy).toEqual(undefined);
        expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
      });
    });
  });
});
