import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { IStrategy, Strategy } from '../strategy.model';
import { StrategyService } from '../service/strategy.service';

@Injectable({ providedIn: 'root' })
export class StrategyRoutingResolveService implements Resolve<IStrategy> {
  constructor(protected service: StrategyService, protected router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IStrategy> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        mergeMap((strategy: HttpResponse<Strategy>) => {
          if (strategy.body) {
            return of(strategy.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new Strategy());
  }
}
