#!/bin/bash -e
#we require a CERTIFICATE_DOMAIN variable
#optionally SANS_DOMAINS variable with comma separated list of domains can be provided
export CERTIFICATE_DOMAIN=$ROOT_DOMAIN

if [ -z "$CERTIFICATE_DOMAIN" ]
then
	echo "\$CERTIFICATE_DOMAIN certificate domain is required"
	exit
fi
if [ -z "$CERTIFICATE_SIGNING_KEY_PASSPHRASE" ]
then
	echo "\$CERTIFICATE_SIGNING_KEY_PASSPHRASE certificate domain is required"
	exit
fi

export CERTIFICATE_DOMAIN_FILENAME=certificate
CERTIFICATE_BASE_DOMAIN=esilab.org
CERTIFICATE_SIGNING_CERT=ca/ca.crt.pem
CERTIFICATE_SIGNING_KEY=ca/ca.key.pem
CERTIFICATE_SIGNING_SUBJ_MINUS_CA=$(openssl x509 -in $CERTIFICATE_SIGNING_CERT -noout -subject -nameopt compat| gawk  '{sub(/subject=/, ""); sub(/\/CN=.*/,""); print}')
CERTIFICATE_SUBJ=$CERTIFICATE_SIGNING_SUBJ_MINUS_CA/CN=$CERTIFICATE_DOMAIN
cp  templates/template.domain.cnf $CERTIFICATE_DOMAIN_FILENAME.cnf

SANS_SEQUENCE=3
if [ ! -z "$SANS_DOMAINS" ]
then
	echo SANS_DOMAINS comma separated list of domains
	for i in $(echo $SANS_DOMAINS | sed "s/,/ /g")
	do
		# call your procedure/other scripts here below
		echo "DNS.$SANS_SEQUENCE	= $i" >> $CERTIFICATE_DOMAIN_FILENAME.cnf
		let "SANS_SEQUENCE+=1" 
	done
fi

rm $CERTIFICATE_DOMAIN_FILENAME.csr > /dev/null 2>&1 || true
rm *.pem > /dev/null 2>&1 || true
rm index.txt* > /dev/null 2>&1 || true
rm serial* > /dev/null 2>&1 || true

touch index.txt
touch index.txt.attr
echo '1000'>serial

openssl req -nodes -out $CERTIFICATE_DOMAIN_FILENAME.csr -newkey rsa:2048 -nodes -keyout $CERTIFICATE_DOMAIN_FILENAME.key.pem -config $CERTIFICATE_DOMAIN_FILENAME.cnf -subj "$CERTIFICATE_SUBJ"

openssl ca -config templates/root.cnf -extensions server_cert -days 3750 -notext -md sha512 -passin pass:$CERTIFICATE_SIGNING_KEY_PASSPHRASE -batch -in $CERTIFICATE_DOMAIN_FILENAME.csr -out $CERTIFICATE_DOMAIN_FILENAME.crt.pem -keyfile $CERTIFICATE_SIGNING_KEY -cert $CERTIFICATE_SIGNING_CERT

if [ $? -ne 0 ]
then
   echo signature problem
   rm $CERTIFICATE_DOMAIN_FILENAME.key.pem
   rm $CERTIFICATE_DOMAIN_FILENAME.csr
   exit
fi

cat $CERTIFICATE_DOMAIN_FILENAME.crt.pem $CERTIFICATE_SIGNING_CERT > chain.$CERTIFICATE_DOMAIN_FILENAME.crt.pem

openssl x509 -in $CERTIFICATE_DOMAIN_FILENAME.crt.pem -text -noout
openssl verify -CAfile $CERTIFICATE_SIGNING_CERT $CERTIFICATE_DOMAIN_FILENAME.crt.pem 

